//  @flow
import 'babel-polyfill';
import React from 'react';
import { render } from 'react-dom';
import { BrowserRouter } from 'react-router-dom';

import type { IConfig } from '../flow/IConfig';

class Client {
  config: IConfig;
  serverProps: { [string]: any };

  constructor(config: IConfig) {
    this.config = config;

    this.getClientProps();
    this.enableCSSInject();
  }

  getClientProps() {
    try {
      const scriptsEl = document.getElementById('app-props');
      let props = scriptsEl ? scriptsEl.textContent : {};

      //  eslint-disable-next-line
      props = props.replace("<![CDATA[", "").replace("]]>", "");
      this.serverProps = JSON.parse(props);
    } catch (err) {
      throw new Error('Error parsing client props');
    }
  }

  enableCSSInject() {
    if (this.config.isDevEnvironment) {
      const head = document.head || document.getElementsByTagName('head')[0];
      const hotInlineStyles = document.createElement('style');
      hotInlineStyles.type = 'text/css';
      head.appendChild(hotInlineStyles);

      require('webpack-hot-middleware/client')
        .subscribe((updateEvent) => {
          if (updateEvent.type === 'css-update') {
            hotInlineStyles.textContent = '';
            hotInlineStyles.appendChild(document.createTextNode(updateEvent.value));
          }
        });
    }
  }

  create(callback: ?Function) {
    const { root: Root } = this.config;

    render(
      <BrowserRouter>
        <Root {...this.serverProps} />
      </BrowserRouter>,
      document.getElementById('root'),
    );

    if (callback && typeof callback === 'function') {
      callback();
    }
  }
}

export default Client;
